<?php

class USIN_Woocommerce_Product_Variations_Loader extends USIN_Standard_Report_Loader {

	protected function load_data(){
		global $wpdb;

		$product_id = $this->get_selected_group_filter('product');
		$product = wc_get_product($product_id);

		if(!$product->is_type('variable')){
			return new WP_Error('report_error', __('This report is not applicable to this product', 'usin'), 'notice');
		}

		$item_meta_table = "{$wpdb->prefix}woocommerce_order_itemmeta";
		$items_table = "{$wpdb->prefix}woocommerce_order_items";

		$filter = $this->getSelectedFilter();
		$orders_join = '';
		$period_condition = USIN_Standard_Report_With_Period_Filter::generate_condition($filter, 'orders.post_date');
		if(!empty($period_condition)){
			$orders_join = " INNER JOIN $items_table AS items ON items.order_item_id = variation_ids.order_item_id 
				INNER JOIN $wpdb->posts AS orders ON orders.ID = items.order_id $period_condition";
		}

		$query = $wpdb->prepare("SELECT COUNT(*) AS $this->total_col, variation_ids.meta_value AS $this->label_col
			FROM $item_meta_table AS variation_ids
			INNER JOIN $item_meta_table AS products ON products.order_item_id = variation_ids.order_item_id " . $orders_join . "
			WHERE variation_ids.meta_key = '_variation_id' AND products.meta_key = '_product_id' AND products.meta_value = %d " . $period_condition . "
			GROUP BY variation_ids.meta_value ORDER BY $this->total_col DESC LIMIT $this->max_items",
			$product_id);

		$data = $wpdb->get_results($query);
		return $this->apply_variation_names($data);
	}

	protected function apply_variation_names($data){
		foreach($data as $row){
			$row->label = $this->get_variation_name($row->label);
		}

		return $data;
	}

	protected function get_variation_name($variation_id){
		$variation = wc_get_product($variation_id);

		if($variation && function_exists('wc_get_formatted_variation')){
			$name = wc_get_formatted_variation($variation, true, true, false);
			if(!empty($name)){
				return $name;
			}
		}

		return "#{$variation_id}";
	}
}